@@ -5,6 +5,8 @@ __all__ = [
     "ParserTree",
 ]
 
+import sys
+
 
 class ParserTreeNode(object):
     def __init__(self, command: str, parser: argparse.ArgumentParser):
@@ -58,12 +60,15 @@ class ParserTree(object):
         # Before Python 3.12 the 'exit_on_error' attribute does not take effect properly
         # if unknown arguments encountered. We have to employ this workaround
         # TODO: remove this workaround when bumping the Python versions to above Python 3.12
-        if self.root_node.parser.exit_on_error:
-            known_args_namespace = self.root_node.parser.parse_args(args)
+        if sys.version_info < (3, 12):
+            if self.root_node.parser.exit_on_error:
+                known_args_namespace = self.root_node.parser.parse_args(args)
+            else:
+                known_args_namespace, unknown_args = self.root_node.parser.parse_known_args(args)
+                if len(unknown_args or []) > 0:
+                    raise argparse.ArgumentError(None, f"unrecognized arguments '{unknown_args}'")
         else:
-            known_args_namespace, unknown_args = self.root_node.parser.parse_known_args(args)
-            if len(unknown_args or []) > 0:
-                raise argparse.ArgumentError(None, f"unrecognized arguments '{unknown_args}'")
+            known_args_namespace = self.root_node.parser.parse_args(args)
 
         command_pairs = []
         namespace = argparse.Namespace()
