@@ -1,7 +1,11 @@
 import unittest
+from collections.abc import Iterable
+from dataclasses import dataclass
+from typing import Self
 
 import ddt
 
+from iker.common.utils.sequtils import Seq
 from iker.common.utils.sequtils import batch_yield
 from iker.common.utils.sequtils import chunk, chunk_between, chunk_with_key, merge_chunks
 from iker.common.utils.sequtils import deduped, grouped
@@ -790,6 +794,26 @@ class SeqUtilsTest(unittest.TestCase):
         self.assertEqual(expect, merge_chunks(data, merge_func=merge_func, drop_exclusive_end=True))
 
 
+@dataclass(frozen=True, eq=True, order=True)
+class Naming(object):
+    serial: int
+    name: str
+    gender: str
+
+
+class NamingSeq(Seq[Naming]):
+
+    def __init__(self, data: Iterable[Naming]):
+        super(NamingSeq, self).__init__(data)
+        self.serials = list(map(lambda x: x.serial, self.data))
+
+    def identity(self) -> Self:
+        return self
+
+    def axis(self) -> list[int]:
+        return self.serials
+
+
 @ddt.ddt
 class SeqTest(unittest.TestCase):
 
@@ -1315,3 +1339,100 @@ class SeqTest(unittest.TestCase):
     def test_zip_fill(self, a, b, fill, expect):
         actual = seq(a).zip_fill(seq(b), fill)
         self.assertEqual(actual.data, expect)
+
+    def test_naming_seq(self):
+        names = [
+            ("Andrew", "M"),
+            ("Alice", "W"),
+            ("Benjamin", "M"),
+            ("Bella", "W"),
+            ("Charles", "M"),
+            ("Charlotte", "W"),
+            ("Daniel", "M"),
+            ("Diana", "W"),
+            ("Ethan", "M"),
+            ("Emily", "W"),
+            ("Felix", "M"),
+            ("Fiona", "W"),
+            ("George", "M"),
+            ("Grace", "W"),
+            ("Henry", "M"),
+            ("Hannah", "W"),
+            ("Isaac", "M"),
+            ("Isabella", "W"),
+            ("Jacob", "M"),
+            ("Julia", "W"),
+            ("Kevin", "M"),
+            ("Katherine", "W"),
+            ("Liam", "M"),
+            ("Lily", "W"),
+            ("Michael", "M"),
+            ("Madison", "W"),
+            ("Nathan", "M"),
+            ("Natalie", "W"),
+            ("Oliver", "M"),
+            ("Olivia", "W"),
+            ("Patrick", "M"),
+            ("Penelope", "W"),
+            ("Quentin", "M"),
+            ("Quinn", "W"),
+            ("Ryan", "M"),
+            ("Rebecca", "W"),
+            ("Samuel", "M"),
+            ("Sophia", "W"),
+            ("Thomas", "M"),
+            ("Tiffany", "W"),
+            ("Ulysses", "M"),
+            ("Ursula", "W"),
+            ("Victor", "M"),
+            ("Victoria", "W"),
+            ("William", "M"),
+            ("Wendy", "W"),
+            ("Xavier", "M"),
+            ("Xenia", "W"),
+            ("Yosef", "M"),
+            ("Yvonne", "W"),
+            ("Zachary", "M"),
+            ("Zoe", "W"),
+        ]
+        naming_seq = NamingSeq(Naming(serial, name, gender) for serial, (name, gender) in enumerate(names))
+
+        self.assertEqual(naming_seq, naming_seq)
+        self.assertEqual(naming_seq.concat(naming_seq).axis(), list(range(0, 52)) + list(range(0, 52)))
+        self.assertEqual(naming_seq.take_left(26).axis(), list(range(0, 26)))
+        self.assertEqual(naming_seq.take_right(26).axis(), list(range(26, 52)))
+        self.assertEqual(naming_seq.reverse().axis(), list(reversed(range(0, 52))))
+        self.assertEqual(naming_seq.distinct().axis(), list(range(0, 52)))
+        self.assertEqual(naming_seq.reduce(lambda x, y: x).axis(), [0])
+        self.assertEqual(naming_seq.reduce(lambda x, y: y).axis(), [51])
+        self.assertEqual(naming_seq.min(lambda x, y: x.serial < y.serial).axis(), [0])
+        self.assertEqual(naming_seq.max(lambda x, y: x.serial > y.serial).axis(), [51])
+        self.assertEqual(naming_seq.filter(lambda x: x.gender == "M").axis(), list(range(0, 52, 2)))
+        self.assertEqual(naming_seq.filter(lambda x: x.gender == "W").axis(), list(range(1, 52, 2)))
+        self.assertEqual(naming_seq.filter_not(lambda x: x.gender == "M").axis(),
+                         naming_seq.filter(lambda x: x.gender == "W").axis())
+        self.assertEqual(naming_seq.filter_not(lambda x: x.gender == "W").axis(),
+                         naming_seq.filter(lambda x: x.gender == "M").axis())
+        self.assertEqual(naming_seq.sort(lambda x: -x.serial).axis(), naming_seq.reverse().axis())
+        self.assertEqual(naming_seq.head().axis(), [0])
+        self.assertEqual(naming_seq.last().axis(), [51])
+        self.assertEqual(naming_seq.init().axis(), list(range(0, 51)))
+        self.assertEqual(naming_seq.tail().axis(), list(range(1, 52)))
+        self.assertEqual(naming_seq.foreach(lambda x: x).data, naming_seq.data)
+        self.assertEqual(naming_seq.union(naming_seq).data, naming_seq.data)
+        self.assertEqual(naming_seq.intersect(naming_seq).data, naming_seq.data)
+
+        self.assertEqual(naming_seq.scan_left(0, lambda x, y: x + y.serial).data,
+                         seq(range(0, 52)).scan_left(0, lambda x, y: x + y).data)
+        self.assertEqual(naming_seq.scan_right(0, lambda x, y: x + y.serial).data,
+                         seq(range(0, 52)).scan_right(0, lambda x, y: x + y).data)
+        self.assertEqual(naming_seq.map(lambda x: x.serial).data, list(range(0, 52)))
+
+        self.assertEqual(naming_seq.fold_left(0, lambda x, y: x + y.serial).data,
+                         seq(range(0, 52)).fold_left(0, lambda x, y: x + y).data)
+        self.assertEqual(naming_seq.fold_right(0, lambda x, y: x + y.serial).data,
+                         seq(range(0, 52)).fold_right(0, lambda x, y: x + y).data)
+
+        self.assertEqual(naming_seq.group(lambda x: x.serial % 2).size, 2)
+        self.assertEqual(naming_seq.group(lambda x: x.gender).size, 2)
+        self.assertEqual(naming_seq.group(lambda x: x.name[0]).size, 26)
